@@ -7,7 +7,7 @@
 # reserved. This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# $Id: Received.pm,v 1.23 2000/04/28 19:58:12 adam Exp $
+# $Id: Received.pm,v 1.28 2003/03/17 23:45:17 adams Exp $
 #
 
 require 5.005;
@@ -23,33 +23,35 @@ use vars qw($VERSION @ISA @EXPORT_OK);
 @ISA = qw(Exporter Mail::Field Mail::Field::Generic);
 @EXPORT_OK = qw(%RC &diagnose);
 
-$VERSION = '0.23';
+$VERSION = '0.24';
 
 =head1 NAME
 
 Mail::Field::Received -- mostly RFC822-compliant parser of Received headers
 
+=head1 SYNOPSIS
+
+  use Mail::Field;
+
+  my $received = Mail::Field->new('Received', $header);
+  my $results = $received->parse_tree();
+  my $parsed_ok = $received->parsed_ok();
+  my $diagnostics = $received->diagnostics();
+
 =head1 DESCRIPTION
 
 I<Don't use this class directly!>  Instead ask Mail::Field for new
 instances based on the field name!
 
-Mail::Field::Received provides subroutines for parsing Received headers
-from e-mails.  It mostly complies with RFC822, but deviates to accomodate
-a number of broken MTAs which are in common use.
+Mail::Field::Received provides subroutines for parsing Received
+headers from e-mails.  It mostly complies with RFC822, but deviates to
+accomodate a number of broken MTAs which are in common use.  It also
+attempts to extract useful information which MTAs often embed within
+the C<(comments)>.
 
 It is a subclass derived from the Mail::Field and Mail::Field::Generic
 classes.
 
-=head1 SYNOPSIS
-
-  use Mail::Field::Received;
-
-  my $received = Mail::Field->new('Received', $header);
-  my $results = $received->parse_tree();
-  my $parsed_ok = $received->parsed_ok();
-  my $diagnostics = $received->diagnostics();
-
 =head1 ROUTINES
 
 =over 4
@@ -153,8 +155,10 @@ $RC{zone}        = qr/(
                         (?:$RC{zone_name})(?:
                       ))/x;
 $RC{hms}         = qr/($RC{TWO_DIGIT}:(\d\d)(?::(\d\d))?)/;
-$RC{month}       = qr/(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)/;
-$RC{week_day}    = qr/(Mon|Tue|Wed|Thu|Fri|Sat|Sun)/;
+# Note: case-insensitivity is not RFC-compliant here, but some MTAs
+# write days/months in all lower case.
+$RC{month}       = qr/(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)/i;
+$RC{week_day}    = qr/(Mon|Tue|Wed|Thu|Fri|Sat|Sun)/i;
 $RC{year}        = qr/((?:19|20)?\d{2}|100)/;  # god-DAMN the incompetence!
 $RC{year_day1}   = qr/(?:$RC{TWO_DIGIT}\s$RC{month})/;
 $RC{year_day2}   = qr/(?:$RC{month}\s$RC{TWO_DIGIT})/;
@@ -289,7 +293,7 @@ sub parse {
               last FROMCOMMENT;
             }
 
-            if ($comment =~ /(HELO|EHLO)\s+($RC{domain})/) {
+            if ($comment =~ /(HELO|EHLO)(?:\s+|=)($RC{domain})/i) {
               # HELO domain is in comments, not outside, so swap
               $self->diagnose("Got `from' $1 domain in comments: $2\n")
                 if $debug >= 3;
@@ -478,7 +482,7 @@ sub parse {
         next TOKEN;
       }
 
-      if ($expecting{after_with}) {
+      if ($expecting{after_with} && $parsed{with}{with}) {
 
         # Microsoft SMTPSVC uses two atoms -- yet /another/ example of
         # Microsoft not following standards ... *gasp*
